<?php

namespace Drupal\spammaster\EventSubscriber;

use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Drupal\Core\Database\Connection;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpFoundation\RedirectResponse;
use Drupal\Core\Session\AccountProxyInterface;
use Drupal\Core\Messenger\MessengerInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Drupal\Core\State\StateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\spammaster\SpamMasterElusiveService;
use Drupal\spammaster\SpamMasterKeyService;
use Drupal\spammaster\SpamMasterMailService;

/**
 * Class firewall subscriber.
 */
class SpamMasterFirewallSubscriber implements EventSubscriberInterface {

  /**
   * The database connection object.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected $connection;

  /**
   * Symfony\Component\HttpFoundation\RequestStack definition.
   *
   * @var \Symfony\Component\HttpFoundation\RequestStack
   */
  protected $requestStack;

  /**
   * Drupal\Core\Session\AccountProxy definition.
   *
   * @var \Drupal\Core\Session\AccountProxy
   */
  protected $currentUser;

  /**
   * The Messenger service.
   *
   * @var \Drupal\Core\Messenger\MessengerInterface
   */
  protected $messenger;

  /**
   * The state.
   *
   * @var \Drupal\Core\State\StateInterface
   */
  protected $state;

  /**
   * The SpamMasterKeyService Service.
   *
   * @var \Drupal\spammaster\SpamMasterKeyService
   */
  protected $keyService;

  /**
   * The SpamMasterElusiveService Service.
   *
   * @var \Drupal\spammaster\SpamMasterElusiveService
   */
  protected $elusiveService;

  /**
   * The SpamMasterMailService Service.
   *
   * @var \Drupal\spammaster\SpamMasterMailService
   */
  protected $mailService;

  /**
   * {@inheritdoc}
   */
  public function __construct(Connection $connection, RequestStack $requestStack, AccountProxyInterface $currentUser, MessengerInterface $messenger, StateInterface $state, SpamMasterKeyService $keyService, SpamMasterElusiveService $elusiveService, SpamMasterMailService $mailService) {
    $this->connection = $connection;
    $this->requestStack = $requestStack;
    $this->currentUser = $currentUser;
    $this->messenger = $messenger;
    $this->state = $state;
    $this->keyService = $keyService;
    $this->elusiveService = $elusiveService;
    $this->mailService = $mailService;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('request_stack'),
      $container->get('messenger'),
      $container->get('current_user'),
      $container->get('state'),
      $container->get('spammaster.key_service'),
      $container->get('spammaster.elusive_service'),
      $container->get('spammaster.mail_service')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function checkForRedirection(RequestEvent $event) {

    $spammaster_ip = $this->requestStack->getCurrentRequest()->getClientIp();
    $spammaster_date = date("Y-m-d H:i:s");
    $spammaster_date_short = date("Y-m-d");
    $spammaster_status = $this->state->get('spammaster.license_status');
    if ('VALID' === $spammaster_status || 'MALFUNCTION_1' === $spammaster_status || 'MALFUNCTION_2' === $spammaster_status) {
      // Exempt whitelist.
      $spammaster_white_query = $this->connection->select('spammaster_white', 'u');
      $spammaster_white_query->fields('u', ['white']);
      $spammaster_white_query->where('(white = :ip)', [':ip' => $spammaster_ip]);
      $spammaster_white_result = $spammaster_white_query->execute()->fetchObject();
      $page = '/firewall';
      // Lets lazy check key and run clean up.
      $spam_master_license_sync_date = $this->state->get('spammaster.spam_master_license_sync_date');
      $spam_master_license_sync_run = $this->state->get('spammaster.spam_master_license_sync_run');
      $spam_master_sync_date_plus_days = date('Y-m-d', strtotime('+2 days', strtotime($spam_master_license_sync_date)));
      if ($spammaster_date_short >= $spam_master_sync_date_plus_days && '1' !== $spam_master_license_sync_run) {
        $updated_values = [
          'spammaster.spam_master_license_sync_date' => $spammaster_date_short,
          'spammaster.spam_master_license_sync_run' => '1',
        ];
        $this->state->setMultiple($updated_values);
        // Spam key controller.
        $spamKey = TRUE;
        $spamCron = 'LAZY';
        $spammaster_key_service = $this->keyService;
        $spammaster_key_service->spamMasterKeyLazy($spamKey, $spamCron);
      }
      // Prepare disc not.
      $spammaster_type = $this->state->get('spammaster.type');
      $spammaster_send = $this->state->get('spammaster.spam_master_disc_not');
      $spammaster_senddate = $this->state->get('spammaster.spam_master_disc_not_date');
      if ('TRIAL' === $spammaster_type || 'FREE' === $spammaster_type) {
        $spammaster_discdate = $spammaster_date_short;
        $spamdesc = FALSE;
        $spamdescper = FALSE;
        $spamcode = FALSE;
        if ('2024-01-01' === $spammaster_date_short) {
          $spamdesc = 'New Year';
          $spamdescper = '50%';
          $spamcode = 'NEWYPRO50';
        }
        if ('2023-02-14' === $spammaster_date_short) {
          $spamdesc = 'Valentines Day';
          $spamdescper = '50%';
          $spamcode = 'VALENTINEPRO50';
        }
        if ('2023-04-01' === $spammaster_date_short) {
          $spamdesc = 'Spring';
          $spamdescper = '25%';
          $spamcode = 'SPRINGRO25';
        }
        if ('2023-06-20' === $spammaster_date_short) {
          $spamdesc = 'Summer';
          $spamdescper = '25%';
          $spamcode = 'SUMMERPRO25';
        }
        if ('2023-08-01' === $spammaster_date_short) {
          $spamdesc = 'August';
          $spamdescper = '25%';
          $spamcode = 'AUGUSTPRO25';
        }
        if ('2023-10-31' === $spammaster_date_short) {
          $spamdesc = 'Halloween';
          $spamdescper = '50%';
          $spamcode = 'HALLOWEENPRO50';
        }
        if ('2023-11-24' === $spammaster_date_short) {
          $spamdesc = 'Black Friday';
          $spamdescper = '25%';
          $spamcode = 'BLACKPRO25';
        }
        if ('2023-11-27' === $spammaster_date_short) {
          $spamdesc = 'Cyber Monday';
          $spamdescper = '50%';
          $spamcode = 'CYBERPRO50';
        }
        if ('2023-12-25' === $spammaster_date_short) {
          $spamdesc = 'Christmas';
          $spamdescper = '25%';
          $spamcode = 'XMASPRO25';
        }
        if ('0' === $spammaster_send && $spammaster_discdate !== $spammaster_senddate && !empty($spamdesc) && !empty($spamdescper) && !empty($spamcode)) {
          $spammaster_mail_service = $this->mailService;
          $spammaster_mail_service->spamMasterDiscNotify($spammaster_discdate, $spamdesc, $spamdescper, $spamcode, $spammaster_send);
        }
      }
      if (!empty($spammaster_white_result)) {

        return;

      }
      else {
        // White negative, proceed to buffer.
        $spammaster_db_ip = $this->connection->select('spammaster_threats', 'u');
        $spammaster_db_ip->fields('u', ['threat']);
        $spammaster_db_ip->where('(threat = :ip)', [':ip' => $spammaster_ip]);
        $spammaster_db_ip_result = $spammaster_db_ip->execute()->fetchObject();
        $spammaster_anonymous = $this->currentUser->isAnonymous();
        if (!empty($spammaster_db_ip_result) && $spammaster_anonymous) {
          $this->connection->insert('spammaster_keys')->fields([
            'date' => $spammaster_date,
            'spamkey' => 'spammaster-firewall',
            'spamvalue' => 'Spam Master: firewall block, Ip: ' . $spammaster_ip,
          ])->execute();

          $spammaster_total_block_count = $this->state->get('spammaster.total_block_count');
          $spammaster_total_block_count_1 = ++$spammaster_total_block_count;
          $this->state->set('spammaster.total_block_count', $spammaster_total_block_count_1);

          $response = new RedirectResponse($page);
          $response->send();

          return;

        }
        else {
          $spamPost = $this->requestStack->getCurrentRequest()->request;
          $spamMethod = $this->requestStack->getCurrentRequest()->getMethod();
          if (!empty($spamPost)) {
            $spamelusive = $spamPost;
            $spammaster_elusive_service = $this->elusiveService;
            $is_json = $spammaster_elusive_service->spamMasterIsJson($spamelusive);
            if ('BAIL' === $is_json) {
              // Requires haf to check.
              return;

            }
            else {
              if ('POST' === $spamMethod && $spammaster_anonymous) {
                $spamelusive = $spamPost;
                $spammaster_elusive_service = $this->elusiveService;
                $isElusive = $spammaster_elusive_service->spamMasterElusiveCheck($spamelusive);
                if ('ELUSIVE' === $isElusive) {

                  $response = new RedirectResponse($page);
                  $response->send();

                  return;

                }
              }
              else {

                return;

              }
            }
          }
          else {
            return;
          }
        }
      }
    }
    else {

      return;

    }
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {

    $events[KernelEvents::REQUEST][] = ['checkForRedirection'];

    return $events;
  }

}
